-- "FrameModified"  	params: ident
-- "FrameRegistered" 	param: category, name, frame, ident

local MAJOR_VERSION = "LibFrameAnchorRegistry-1.0"
local MINOR_VERSION = 5

if not LibStub then error(MAJOR_VERSION .. " requires LibStub.") end
local lib = LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
if not lib then return end

lib.callbacks = lib.callbacks or LibStub("CallbackHandler-1.0"):New(lib)

lib.frames = lib.frames or {}

--- Register a new anchor freame
-- @param category The category that this frame will belong to. To make sub-categories add them together separated with '\001' ('MyCat\001MySubCat').
-- @param name The name of the anchor.
-- @param ident Unique identifyer for the frame.
-- @param frame The actual frame that is to be the anchor.
-- @usage lib:RegisterAnchor("NewCategory", "TheAnchor", frame)
function lib:RegisterAnchor(category, name, ident, frame)
	if not category then error("Usage: LibFrameAnchor:RegisterAnchor(category, name, ident, frame): 'category' - valid category expected.", 2) end
	if not name or type(name) ~= "string" then error("Usage: LibFrameAnchor:RegisterAnchor(category, name, ident, frame): 'name' - name expected.", 2) end
	if not ident or type(ident) ~= "string" then error("Usage: LibFrameAnchor:RegisterAnchor(category, name, ident, frame): 'ident' - unique identifyer expected", 2) end
	if not frame or not frame.GetObjectType then error("Usage: LibFrameAnchor:RegisterAnchor(category, name, ident, frame): 'frame' - frame type expected.", 2) end
	
	for i, data in pairs(lib.frames) do		
		if data.ident == ident then
			lib.frames[i] = nil
		end
	end
	tinsert(lib.frames, {category = category, name = name, ident = ident, frame = frame})	
	
	lib.callbacks:Fire("FrameRegistered", category, name, frame, ident)
end

--- Unregister an anchor
-- @param ident Identification for the frame to unregister.
function lib:UnregisterAnchor(ident)
	if not ident or type(ident) ~= "string" then error("Usage: LibFrameAnchorRegistry:UnregisterAnchor(ident): 'ident' - string expected", 2) end
	
	for i, data in pairs(lib.frames) do
		if data.ident == ident then
			lib.callbacks:Fire("FrameUnregistered", ident)
			tremove(lib.frames, i)
			break
		end
	end
end

--- Get a full list of all anchors.
-- @param frame The frame that is going to be anchored. If passed it wont show up itself in the lists if it is also registered as an anchor.
-- @return table Format: ret['CategoryName'][1..n] { name, disabled, frame }. Disabled indicates that this frame cannot be anchored to as it would create a circular reference.
function lib:GetAnchors(frame)
	if not frame or not frame.GetObjectType then
		error("Usage: LibFrameAnchor:GetAnchors(frame): 'frame' - frame type expected.", 2)
	end
	
	local ret = {}
	
	for i, data in pairs(lib.frames) do
		if data.frame ~= frame then
			tinsert(ret, {
				category = data.category,
				name = data.name,
				frame = data.frame,
				ident = data.ident,
				disabled = FindReference(frame, data.frame),
			})
			-- Dummy index for AceConfig dropdown to work properly when setting value.
			ret[data.ident] = true
		end
	end
		
	return ret
end

--- Get an anchor frame
-- @param ident The unique identifyer of the frame.
-- @return frame The anchor frame requested.
function lib:GetAnchor(ident)	
	for _, data in pairs(lib.frames) do
		if data.ident == ident then
			return data.frame
		end
	end
	
	return UIParent
end

--- Notify the library that a frame that is registered as anchor have changed in some way.
-- Triggers FrameModified callback for addons listening to it so they can act properly on it.
-- @param category The category.
-- @param name Name of the anchor.
function lib:FrameModified(ident)
	for _, data in pairs(lib.frames) do
		if data.ident == ident then
			lib.callbacks:Fire("FrameModified", ident)
		end
	end
end

FindReference = function(frame, target)
	local n = target:GetNumPoints()
	for i = 0, n do
		local point, rel, relPoint, x, y = target:GetPoint(i)
		if not rel then
			rel = target:GetParent()
		end
		if rel == frame then
			return true
		elseif rel then
			return FindReference(frame, rel)
		end
	end
	
	return false
end

local AnchorList = {}	
function lib:GetAnchorList()		
	AnchorList = AnchorList or {}
	AnchorList["TOPLEFT"] = "Top left"
	AnchorList["TOP"] = "Top"
	AnchorList["TOPRIGHT"] = "Top Right"
	AnchorList["RIGHT"] = "Right"
	AnchorList["BOTTOMRIGHT"] = "Bottom right"
	AnchorList["BOTTOM"] = "Bottom"
	AnchorList["BOTTOMLEFT"] = "Bottom left"
	AnchorList["LEFT"] = "Left"
	AnchorList["CENTER"] = "Center"
	
	return AnchorList
end


lib:RegisterAnchor("Blizzard", "Minimap", "BlizzMinimap", Minimap)
lib:RegisterAnchor("Blizzard", "Minimap Cluster", "BlizzMinimapCluster", MinimapCluster)
lib:RegisterAnchor("Blizzard", "Screen (UIParent)", "BlizzUIParent", UIParent)
lib:RegisterAnchor("Blizzard", "UnitFrame - Player", "BlizzUnitFramePlayer", PlayerFrame)
lib:RegisterAnchor("Blizzard", "UnitFrame - Target", "BlizzUnitFrameTarget", TargetFrame)
lib:RegisterAnchor("Blizzard", "World Frame", "BlizzWorldFrame", WorldFrame)
lib:RegisterAnchor("Blizzard", "Chat Frame", "BlizzChatFrame", ChatFrame1)
lib:RegisterAnchor("Blizzard", "Quest Tracker", "BlizzardWatchFrame", WatchFrame)

lib.frame = lib.frame or CreateFrame("Frame")
lib.frame:Hide()
lib.frame:SetScript("OnEvent", function(self, event, ...) 
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")

	if PitBull4_Frames_player 		then lib:RegisterAnchor("PitBull4", "Player", "PibBull4Player", PitBull4_Frames_player) end
	if PitBull4_Frames_target 		then lib:RegisterAnchor("PitBull4", "Target", "PibBull4Target", PitBull4_Frames_target) end
	if PitBull4_Frames_pet 			then lib:RegisterAnchor("PitBull4", "Pet", "PibBull4Pet", PitBull4_Frames_pet) end
	if PitBull4_Frames_targettarget then lib:RegisterAnchor("PitBull4", "Target of Target", "PibBull4TargetOfTarget", PitBull4_Frames_targettarget) end
	if PitBull4_Frames_focus 		then lib:RegisterAnchor("PitBull4", "Focus", "PibBull4Focus", PitBull4_Frames_focus) end
	if PitBull4_Frames_focustarget	then lib:RegisterAnchor("PitBull4", "Focus Target", "PibBull4FocusTarget", PitBull4_Frames_focustarget) end
end)
lib.frame:RegisterEvent("PLAYER_ENTERING_WORLD")

local FindReference