local AceGUI = LibStub("AceGUI-3.0")

-- Lua APIs
local pairs, assert, type = pairs, assert, type

-- WoW APIs
local CreateFrame, UIParent = CreateFrame, UIParent

local Type = "UIF-Frame"
local Version = 12

local Backdrop  = {
	--bgFile = [[Interface\ChatFrame\ChatFrameBackground]],
	edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
	tile = true, tileSize = 16, edgeSize = 16,
	insets = { left = 2, right = 2, top = 2, bottom = 2 }
}

local BackgroundColor = {0, 0, 0, 0.6}
local BorderColor = {0.33, 0.33, 0.33, 1}
local Font, FontSize, FontFlags = GameFontNormal:GetFont()
local fr, fg, rb, ra = GameFontNormal:GetTextColor()

--[[-----------------------------------------------------------------------------
Scripts
-------------------------------------------------------------------------------]]
local function Frame_OnClose(frame)
	frame.obj:Fire("OnClose")
end

local function CloseButton_Click(this)
	this.obj:Hide()
end

local function Frame_OnMouseDown(frame)
	frame:StartMoving()
	AceGUI:ClearFocus()
end

local function Frame_OnMouseUp(frame)
	local self = frame.obj
	frame:StopMovingOrSizing()
	local status = self.status or self.localstatus
	status.width = frame:GetWidth()
	status.height = frame:GetHeight()
	status.top = frame:GetTop()
	status.left = frame:GetLeft()
end

local function StopSizing(mover)
	local frame = mover:GetParent()
	frame:StopMovingOrSizing()
	local self = frame.obj
	local status = self.status or self.localstatus
	status.width = frame:GetWidth()
	status.height = frame:GetHeight()
	status.top = frame:GetTop()
	status.left = frame:GetLeft()
end

local function SizerWH_OnMouseDown(frame)
	frame:GetParent():StartSizing("BOTTOMRIGHT")
	AceGUI:ClearFocus()
end
	
--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
	["OnAcquire"] = function(self)
		local frame = self.frame
		local r, g, b, a = unpack(BackgroundColor)
		self.background:SetTexture(r, g, b, a)
		self.title:SetVertexColor(r + 0.02, g + 0.02, b + 0.01, a)		
		r, g, b, a = unpack(BorderColor)
		frame:SetBackdropBorderColor(r, g, b, a)	
		
		self.icon:SetTexture()
		self.icon:SetWidth(0.0001)
		
		self.title.text:SetText()
		self.title:SetHeight(0.0001)
		self.title.text:SetTextColor(fr, fg, fb, fa)
		self.title.text:SetFont(Font, FontSize, FontFlags)
		self.title:Hide()
		
		self.statusbar.text:SetText()
		self.statusbar:SetHeight(self.title.text:GetStringHeight())
		self.statusbar:SetBackdropBorderColor(r, g, b, a)
		self.statusbar.text:SetTextColor(fr, fg, fb, fa)
		self.statusbar.text:SetFont(Font, FontSize, FontFlags)
		
		self:ApplyStatus()
	end,
	
	["OnRelease"] = function(self)
		self.status = nil
		wipe(self.localstatus)
	end,
	
	["Show"] = function(self)
		self.frame:Show()
	end,
	
	["Hide"] = function(self)
		self.frame:Hide()
	end,
	
	["SetStatusTable"] = function(self, status)
		assert(type(status) == "table")
		self.status = status
		self:ApplyStatus()
	end,
	
	["ApplyStatus"] = function(self)
		local status = self.status or self.localstatus
		local frame = self.frame
		self:SetWidth(status.width or 700)
		self:SetHeight(status.height or 500)
		frame:ClearAllPoints()
		if status.top and status.left then
			frame:SetPoint("TOP", UIParent, "BOTTOM", 0, status.top)
			frame:SetPoint("LEFT", UIParent, "LEFT", status.left, 0)
		else
			frame:SetPoint("CENTER")
		end
	end,
	
	["SetTitle"] = function(self, text)
		self.title.text:SetText(text)
		self.title:SetHeight(self.title.text:GetStringHeight() + 10)		
		self.title:Show()
		self.content:SetPoint("TOPLEFT", self.frame, 10, -self.title:GetHeight() - 10)
	end,
	
	["SetBackgroundColor"] = function(self, color)
		local r, g, b, a = unpack(color)
		self.background:SetTexture(r, g, b, a)
		self.title:SetVertexColor(r, g, b, 1)
	end,
	
	["SetBorderColor"] = function(self, color)
		local r, g, b, a = unpack(color)
		self.frame:SetBackdropBorderColor(r, g, b, a)
		self.statusbar:SetBackdropBorderColor(r, g, b, a)
	end,
	
	["SetFont"] = function(self, font, size, flags)
		self.title.text:SetFont(font, size, flags)
		self.statusbar.text:SetFont(font, 14, flags)
	end,
	
	["SetFontColor"] = function(self, color)
		self.title.text:SetTextColor(unpack(color))
	end,
	
	["SetIcon"] = function(self, icon, tl, tr, tt, tb)
		self.icon:SetTexture(icon)
		if tl and tr and tt and tb then
			self.icon:SetTexCoord(tl, tr, tt, tb)
		end
		if self.icon:GetTexture() then
			if self.title.text:GetText() then
				local w = self.title.text:GetStringHeight()
				self.icon:SetHeight(w)
				self.icon:SetWidth(w)
			else
				self.icon:SetWidth(30)
				self.icon:SetHeight(30)
				self.title:SetHeight(35)		
				self.title:Show()
				self.content:SetPoint("TOPLEFT", self.frame, 10, -self.title:GetHeight() - 10)
			end
		end
	end,
	
	["SetStatusText"] = function(self, text)
		self.statusbar.text:SetText(text)
		self.statusbar:SetHeight(self.statusbar.text:GetStringHeight() + 10)
	end,
}

local function Constructor()
	Backdrop.edgeFile = GetUIFPath().."\\textures\\border2.tga"	
	local frame = CreateFrame("Frame", nil, UIParent)
	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetResizable(true)
	frame:SetFrameStrata("FULLSCREEN_DIALOG")	
	frame:SetMinResize(400, 200)
	frame:SetToplevel(true)
	frame:SetBackdrop(Backdrop)		
	frame:SetScript("OnHide", Frame_OnClose)
	frame:SetScript("OnMouseDown", Frame_OnMouseDown)
	frame:SetScript("OnMouseUp", Frame_OnMouseUp)

	Backdrop.edgeFile = GetUIFPath().."\\textures\\border1.tga"
	Backdrop.insets = {left = 1, right = 1, top = 1, bottom = 1}
	local title = frame:CreateTexture(nil, "ARTWORK", nil, 1)
	title:SetTexture(GetUIFPath().."\\textures\\titleBackground.tga")
	title:SetHorizTile(true)	
	title:SetPoint("TOPLEFT", 3, -3)
	title:SetPoint("TOPRIGHT", -3, -3)
	
	local icon = frame:CreateTexture(nil, "ARTWORK", nil, 2)
	icon:SetPoint("LEFT", title, 10, 0)
	icon:SetWidth(0.001)
	icon:SetHeight(0.001)
	
	local titleString = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	titleString:SetPoint("LEFT", icon, "RIGHT", 10, 0)
	title.text = titleString	
	
	local closebutton = CreateFrame("Button",nil,frame,"UIPanelButtonTemplate")
	closebutton:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT", -10, 10)
	closebutton:SetScript("OnClick", CloseButton_Click)
	closebutton:SetHeight(22)
	closebutton:SetWidth(100)
	closebutton:SetText(CLOSE)
	
	Backdrop.edgeFile = GetUIFPath().."\\textures\\border4.tga"
	local statusbar = CreateFrame("Frame", nil, frame)	
	statusbar:SetPoint("BOTTOMLEFT", 10, 10)
	statusbar:SetPoint("RIGHT", closebutton, "LEFT", -5, 0)
	statusbar:SetBackdrop(Backdrop)
	statusbar:SetBackdropColor(0.1, 0.1, 0.1, 0.5)
	statusbar:SetBackdropBorderColor(0.4, 0.4, 0.4)
	
	local statusbarString = statusbar:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	statusbarString:SetPoint("LEFT", statusbar)
	statusbar.text = statusbarString
	
	local background = frame:CreateTexture(nil, "ARTWORK")
	background:SetPoint("TOPLEFT", title, "BOTTOMLEFT")
	background:SetPoint("BOTTOMRIGHT", frame, -3, 3)	
	
	local sizer = CreateFrame("Button", nil, frame)
	sizer:SetNormalTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Up")
	sizer:SetHighlightTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Highlight")
	sizer:SetPushedTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Down")
	sizer:SetPoint("BOTTOMRIGHT")
	sizer:SetSize(16, 16)
	sizer:SetScript("OnMouseDown",SizerWH_OnMouseDown)
	sizer:SetScript("OnMouseUp", StopSizing)
	
	local content = CreateFrame("Frame", nil, frame)
	content:SetPoint("TOPLEFT", frame, "TOPLEFT",  10, -10)
	content:SetPoint("BOTTOM", statusbar, "TOP", 0, 10)
	content:SetPoint("RIGHT", -10, 0)
	
	local widget = {
		localstatus = {},
		frame = frame,
		icon = icon,
		background = background,
		statusbar = statusbar,
		title = title,
		content = content,
		type = Type,
	}
	for method, func in pairs(methods) do
		widget[method] = func
	end
	
	frame.obj, closebutton.obj, sizer.obj = widget, widget, widget
	
	AceGUI:RegisterAsContainer(widget)
	return widget	
end

AceGUI:RegisterWidgetType(Type,Constructor,Version)
